const fs = require('fs');
const path = require('path');
const childProcess = require('child_process');
const VideoGravacoes = require('../../../models/VideoGravacoes');

exports.uploadVideo = async (request, reply) => {
    const { callid } = request.params;
    const { file, type } = request.payload;

    console.log('Recebendo arquivo de video', callid, type);

    const date = new Date();
    const dir = `/gravacoes/Videos/${date.getFullYear()}/${date.getMonth() + 1}/${date.getDate()}/${callid}`;
    if (!fs.existsSync(path.resolve(dir))) {
        fs.mkdirSync(path.resolve(dir), { recursive: true });
    }
    try {
        const newName = file.hapi.filename;
        // const newName = name.substring(0, name.lastIndexOf(".")) + uuid() + name.substring(name.lastIndexOf("."));
        const file_path = path.resolve(dir, newName);
        const writeStream = fs.createWriteStream(file_path);
        writeStream.on("error", function (err) {
            fs.unlinkSync(file_path);
            resolve({ ...file, error: err, size: file.bytes });
        });
        file.pipe(writeStream);
        file.on("end", async function (err) {
            if (err) {
                fs.unlinkSync(file_path);
                resolve({ ...file, error: err, size: file.bytes });
            } else {
                new VideoGravacoes({
                    callid,
                    link: dir,
                }).insert();
                reply({ path: file_path, size: file.bytes });
            }
        });
        
    } catch (e) {
        reply(Boom.badData(e));
    }
}

exports.getVideo = async (request, reply) => {
    const { callid, arquivo } = request.params;
    console.log('Requisitando video', callid, arquivo)
    const video = await VideoGravacoes.fetchByCallid(callid)
    if (!video) {
        return reply(Boom.notFound('Video não encontrado'));
    }
    const file_path = path.resolve(video.link, arquivo);
    console.log('Arquivo', file_path)
    if (!fs.existsSync(file_path)) {
        if (arquivo === 'output.mp4') {
            if (!fs.existsSync(path.resolve(video.link, 'remoto.webm')) || !fs.existsSync(path.resolve(video.link, 'local.webm'))) {
                return reply(Boom.notFound('Video não encontrado'));
            }
            // ffmpeg
            const child = childProcess.spawn('/usr/bin/ffmpeg', [
                '-i', path.resolve(video.link, 'remoto.webm'),
                '-i', path.resolve(video.link, 'local.webm'),
                '-filter_complex', '[1:v]scale=240:160 [ovrl],[0:v][ovrl]overlay=0:main_h-overlay_h,[0:a][1:a]amix',
                file_path,
            ]);
            child.on('error', () => {
                return reply(Boom.badData('Erro ao gerar video'));
            });
            child.on('close', code => {
                if (code !== 0) {
                    return reply(Boom.badData('Erro ao gerar video'));
                } else {
                    reply.file(file_path);
                }
            })
            // child.on('exit', () => {
            //     return reply.file(file_path);
            // });
        } else {
            reply(Boom.notFound('Video não encontrado'));
        }
    } else {
        try {
            console.log('Enviando arquivo', file_path)
            reply.file(file_path, {
                name: arquivo,
                confine: false,
            });
        } catch (error) {
            reply(Boom.badData(error));
        }
    }
}